

import java.io.IOException;
import java.nio.file.Paths;

public class CompilerTest {

	private static boolean failed = false;

	private static boolean notEqual(String a, String b) {
		if (a == null || b == null)
			return false;
		return !a.equals(b);
	}

	public static void main(String[] args) throws IOException {
		pushAndVerify("1.1", 1);
		pushAndVerify("3.0", 2);
		pushAndVerify("5.4", 3);

		popAndVerify("5.4", 2);
		popAndVerify("3.0", 1);
		popAndVerify("1.1", 0);

		String header1 = "public class Test {\n"
				+ "\tpublic static void main(String[] args) {\n"
				+ "\t\tif(args.length < 0) {\n"
				+ "\t\t\tSystem.out.println(\"Missing arguments\");\n"
				+ "\t\t\treturn;\n" + "\t\t}\n";

		if (notEqual(header1, Compiler.generateHeader("Test", 0))) {
			System.out.println("Failed to verify first header");
			failed = true;
		}

		String header2 = "public class WithArguments {\n"
				+ "\tpublic static void main(String[] args) {\n"
				+ "\t\tif(args.length < 2) {\n"
				+ "\t\t\tSystem.out.println(\"Missing arguments\");\n"
				+ "\t\t\treturn;\n" + "\t\t}\n"
				+ "\t\tdouble arg0 = Double.parseDouble(args[0]);\n"
				+ "\t\tdouble arg1 = Double.parseDouble(args[1]);\n";

		if (notEqual(header2, Compiler.generateHeader("WithArguments", 2))) {
			System.out.println("Failed to verify second header");
			failed = true;
		}

		popAndVerify("arg1", 1);
		popAndVerify("arg0", 0);

		runCommandAndVerify("2.2", ".*2\\.2\\d*", 1);
		runCommandAndVerify("2", ".*2(\\.0.*+)?", 2);

		Compiler.pop();
		Compiler.pop();

		String footer1 = "\t\tdouble result = 1.0;\n"
				+ "\t\tSystem.out.println(result);\n" + "\t}\n" + "}\n";

		pushAndVerify("1.0", 1);
		if (notEqual(footer1, Compiler.generateFooter())) {
			System.out.println("Failed to verify first footer");
			failed = true;
		}
		popAndVerify("1.0", 0);

		String footer2 = "\t\tdouble result = (1.0 * 20.1);\n"
				+ "\t\tSystem.out.println(result);\n" + "\t}\n" + "}\n";

		pushAndVerify("(1.0 * 20.1)", 1);
		if (notEqual(footer2, Compiler.generateFooter())) {
			System.out.println("Failed to verify first footer");
			failed = true;
		}
		popAndVerify("(1.0 * 20.1)", 0);

		String vat = Compiler.generateSourceCode(Paths.get("resources",
				"Vat.calc"));
		if (vat == null || !vat.matches(buildSourceExpression("Vat"))) {
			System.out.println("Failed to generate source for Vat");
			failed = true;
		}
		String halfCircle = Compiler.generateSourceCode(Paths.get("resources",
				"HalfCircleArea.calc"));
		if (halfCircle == null || !halfCircle.matches(buildSourceExpression("HalfCircleArea"))) {
			System.out.println("Failed to generate source HalfCircleArea");
			failed = true;
		}

		if (!failed)
			System.out.println("Q1b Passed");
	}

	private static String buildSourceExpression(String className) {
		return "public class " + className + " \\{\n"
				+ "\tpublic static void main\\(String\\[\\] args\\) \\{\n"
				+ "\t\tif\\(args\\.length < 1\\) \\{\n"
				+ "\t\t\tSystem.out.println\\(\"Missing arguments\"\\);\n"
				+ "\t\t\treturn;\n" + "\t\t\\}\n"
				+ "\t\tdouble arg0 = Double.parseDouble\\(args\\[0\\]\\);\n"
				+ "\t\tdouble result = .*;\n"
				+ "\t\tSystem.out.println\\(result\\);\n" + "\t\\}\n" + "\\}\n";

	}

	private static void runCommandAndVerify(String cmd, String top, int size) {
		String ret = Compiler.processCommand(cmd);

		if (ret == null || !ret.matches(top)) {
			System.out.println("Failed to verify command (" + cmd + ")");
			failed = true;
		}

		verifySize(size);
	}

	private static void popAndVerify(String value, int size) {
		String top = Compiler.pop();

		if (notEqual(top, value)) {
			System.out.println("Failed to verify pop (" + value + ")");
			failed = true;
		}

		verifySize(size);
	}

	private static void pushAndVerify(String value, int size) {
		Compiler.push(value);
		if (notEqual(Compiler.top(), value)) {
			System.out.println("Failed to verify push ("
					+ String.valueOf(value) + ")");
			failed = true;
		}

		verifySize(size);
	}

	private static void verifySize(int size) {
		if (Compiler.size() != size) {
			System.out.println("Stack size should be " + size);
			failed = true;
		}
	}
}
